unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, TeeProcs, TeEngine, Chart, Series, VclTee.TeeGDIPlus,
  VCLTee.TeeAnimations, Vcl.ComCtrls, VCLTee.TeeTools;

type
  TReportGraphic = class
  private
    FRect: TRect;
    FBitmap: TBitmap;
    FMetafile: TMetafile;
    FUseBitmap: boolean;
    function GetGraphic: TGraphic;
  public
    constructor Create;
    destructor Destroy; override;
    property Rect: TRect read FRect write FRect;
    property Bitmap: TBitmap read FBitmap;
    property Metafile: TMetafile read FMetafile;
    property UseBitmap: boolean read FUseBitmap write FUseBitmap;
    property Graphic: TGraphic read GetGraphic;
  end;

  TForm10 = class(TForm)
    pnlTop: TPanel;
    Splitter1: TSplitter;
    pnlDuplicate: TPanel;
    Image1: TImage;
    Panel3: TPanel;
    Chart1: TChart;
    annPayout1: TAnnotationTool;
    chrtlineLifeExpect1: TColorLineTool;
    annLifeExpect1: TAnnotationTool;
    pnlHeader: TPanel;
    chkUseBitmap: TCheckBox;
    lblResult: TLabel;
    chkOpenImage: TCheckBox;
    procedure chkUseBitmapClick(Sender: TObject);
    procedure Splitter1Moved(Sender: TObject);
  private
    procedure UpdateDuplicateImage;
//    procedure PreparePrint;
    { Private declarations }
  protected
    procedure DoShow; override;
    procedure Resize; override;
    function GetFileName(const aExt: string): string;

  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;

  end;

var
  Form10: TForm10;

implementation

uses
  Winapi.ShellAPI;

{$R *.dfm}

function GetIncrementedFileName(const aFileMask: String): String;
var
  lIdx: Integer;
  lFileName: string;
begin
  lIdx := 0;
  repeat
    inc(lIdx);
    lFileName := Format(aFileMask, [lIdx]);
  until not FileExists(lFileName);
  result := lFileName;
end;


{ TReportGraphics }
constructor TReportGraphic.Create;
begin
  FUseBitmap := False;
  FBitmap := TBitmap.Create;
  FMetafile := TMetafile.Create;
end;

destructor TReportGraphic.Destroy;
begin
  FBitmap.Free;
  FMetafile.Free;
  inherited;
end;


function TReportGraphic.GetGraphic: TGraphic;
begin
  if FUseBitmap then
    result := FBitmap
  else
    result := FMetafile;    
end;


constructor TForm10.Create(aOwner: TComponent);
var
  I: Integer;
begin
  inherited;

  for I := 0 to Chart1.SeriesCount-1 do
  begin
   Chart1.Series[I].FillSampleValues(10);
   Chart1.Series[I].Visible := false;
  end;
end;

procedure TForm10.DoShow;
begin
  inherited;

  Chart1.Height := (ClientHeight div 5) * 2;
  UpdateDuplicateImage;
end;

function TForm10.GetFileName(const aExt: string): string;
var
  lMask: string;
begin
  lMask := ExtractFilePath(ParamStr(0)) + 'Image%d' + aExt;
  result := GetIncrementedFileName(lMask);
end;

procedure AssignChartToGraphic(aChart: TChart; const aRect: TRect; aGraphic: TGraphic);
var
  lHoldShadow: boolean;
  I: Integer;
  lBoolArray: array of boolean;
  lHoldBevel: TPanelBevel;
  lHoldColor: TColor;
  lHoldGradientVisible: boolean;
  lMetafile: TMetafile;
  lBmp: TBitmap;
begin

  lHoldGradientVisible := aChart.Gradient.Visible;
  lHoldColor := aChart.Color;
  lHoldBevel := aChart.BevelOuter;
  lHoldShadow := aChart.Shadow.Visible;


  try
    SetLength(lBoolArray, aChart.SeriesCount);
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
      begin
        lBoolArray[I] := TPieSeries(aChart.Series[I]).Circled;
        TPieSeries(aChart.Series[I]).Circled := True;
      end
      else
        lBoolArray[I] := True;


    aChart.Gradient.Visible  := false;
    aChart.Color := clWhite;
    aChart.BevelOuter := bvNone;
    aChart.Shadow.Visible := False;

    if aGraphic is TMetafile then
    begin
      lMetafile := aChart.TeeCreateMetafile(False, aRect);
      try
        aGraphic.Assign(lMetafile);
      finally
        lMetafile.Free;
      end;
    end
    else
    begin
      lBmp := aChart.TeeCreateBitmap(clWhite, aRect);
      try
        aGraphic.Assign(lBmp);
      finally
        lBmp.Free;
      end;
    end;
  finally
    aChart.Shadow.Visible := lHoldShadow;
    aChart.Gradient.Visible := lHoldGradientVisible;
    aChart.Color := lHoldColor;
    aChart.BevelOuter := lHoldBevel;
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
        TPieSeries(aChart.Series[I]).Circled := lBoolArray[I];
  end;
end;



procedure TForm10.chkUseBitmapClick(Sender: TObject);
begin
  UpdateDuplicateImage;
end;

procedure TForm10.UpdateDuplicateImage;
var
  lRG: TReportGraphic;
  lChart: TChart;
  lFileName: string;
begin
  lRG := TReportGraphic.Create;
  lRG.Rect := Rect(1, 1, Trunc(Image1.Width), Trunc(Image1.Height));

  lChart := Chart1;
  lChart.View3D := false;
//  lChart.Series[0].Marks.Visible := False;
//  TPieSeries(lChart.Series[0]).PiePen.Color := clWhite;
  lChart.Title.Visible := False;
  lChart.Border.Visible := False;
  lChart.Shadow.Visible := False;

  lRG.UseBitmap := chkUseBitmap.Checked;

  if lRG.UseBitmap then
  begin
    AssignChartToGraphic(lChart, lRG.Rect, lRG.Bitmap);
    Image1.Picture.Bitmap.Assign(lRG.Bitmap);
    lFileName := GetFileName('.BMP');
  end
  else
  begin
    AssignChartToGraphic(lChart, lRG.Rect, lRG.Metafile);
    Image1.Picture.Metafile.Assign(lRG.Metafile);
    lFileName := GetFileName('.WMF');
  end;

  if chkOpenImage.Checked then
  begin
    lRG.Metafile.SaveToFile(lFileName);
    ShellExecute(Application.Handle, 'open', PChar(lFileName), '', Pchar(lFileName), SW_SHOW);
  end;

  lblResult.Caption := Format(
    'Requested Size: (%d, %d);  Result Size: (%d, %d)',
    [lRG.Rect.Width, lRG.Rect.Height, lRG.Graphic.Width, lRG.Graphic.Height]
    );

  lRG.Free;
end;


{procedure TForm10.PreparePrint;
var
  lChart: TChart;
  lRG: TReportGraphic;
  lFIleName: string;
begin

  //
  // Pie Chart Image
  //

  lRG := TReportGraphic.Create('Test', imgPieChart);
  lRG.Rect := Rect(1, 1, Trunc(imgPieChart.spWidth), Trunc(imgPieChart.spHeight));

  lChart := Chart1;
  lChart.View3D := false;
  lChart.Title.Visible := False;
  lChart.Border.Visible := False;
  lChart.Shadow.Visible := False;
  lRG.UseBitmap := chkUseBitmap.Checked;

  if lRG.UseBitmap then
  begin
    AssignChartToGraphic(lChart, lRG.Rect, lRG.Bitmap);
    imgPieChart.Picture.Bitmap.Assign(lRG.Bitmap);
    lFIleName := GetFileName('.BMP');
    lRG.Bitmap.SaveToFile(lFileName);
  end
  else
  begin
    AssignChartToGraphic(lChart, lRG.Rect, lRG.Metafile);
    imgPieChart.Picture.Metafile.Assign(lRG.Metafile);
    lRG.Metafile.SaveToFile(GetFileName('.WMF'));
    lFileName := GetFileName('.WMF');
    lRG.Metafile.SaveToFile(lFileName);
  end;
  ShellExecute(Application.Handle, 'open', PChar(lFileName), '', Pchar(lFileName), SW_SHOW);

  lRG.Free;

  ppReport1.Print;
end;}

procedure TForm10.Resize;
begin
  inherited;
  UpdateDuplicateImage;

end;

procedure TForm10.Splitter1Moved(Sender: TObject);
begin
  UpdateDuplicateImage;
end;

end.
